/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * Copyright (c) 2017 STMicroelectronics International N.V. 
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other 
  *    contributors to this software may be used to endorse or promote products 
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this 
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under 
  *    this license is void and will automatically terminate your rights under 
  *    this license. 
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS" 
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f4xx_hal.h"
#include "dma.h"
#include "fatfs.h"
#include "i2s.h"
#include "rtc.h"
#include "sdio.h"
#include "usart.h"
#include "usb_device.h"
#include "usb_host.h"
#include "gpio.h"
#include "fmc.h"
#include "player.h"
#include <iostream>
#include <sidplay/sidplay2.h>
#include <sidplay/builders/resid.h>
#include "libsidplay/include/sidplay/sidplay2.h"
SDRAM_HandleTypeDef hsdram;
FMC_SDRAM_TimingTypeDef SDRAM_Timing;
FMC_SDRAM_CommandTypeDef command;

#define REFRESH_COUNT       ((uint32_t)0x056A)   /* SDRAM refresh counter (90MHz SDRAM clock) */

using namespace __sidplay2__;
using namespace SIDPLAY2_NAMESPACE;
volatile uint8_t sdram_init = 0;



extern "C" caddr_t _sbrk(int incr)
{
	extern char __heap_start; // Defined by the linker.
	  extern char __heap_limit; // Defined by the linker.
	  extern char __sdram_start;
	  extern char __sdram_limit;

	if (sdram_init) {
		static char *sdram_heap_end;
		static char *sdram_heap_limit = &__sdram_limit;
		char *sdram_prev_heap_end;

		if (sdram_heap_end == 0)
			sdram_heap_end = &__sdram_start;

		sdram_prev_heap_end = sdram_heap_end;
		if (sdram_heap_end + incr > sdram_heap_limit)
		{
			errno = ENOMEM; // not enough memory
			return (caddr_t) -1;
		}

		sdram_heap_end += incr;

		return (caddr_t) sdram_prev_heap_end;


	}

	static char *heap_end;
    static char *heap_limit = &__heap_limit;


	char *prev_heap_end;

	if (heap_end == 0)
		heap_end = &__heap_start;

	prev_heap_end = heap_end;
	if (heap_end + incr > heap_limit)
	{
		errno = ENOMEM; // not enough memory
		return (caddr_t) -1;
	}

	heap_end += incr;

	return (caddr_t) prev_heap_end;
}



/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
void MX_USB_HOST_Process(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

/* USER CODE END 0 */


__IO uint32_t uwWriteReadStatus = 0;

void SDRAM_Initialization_Sequence(SDRAM_HandleTypeDef *hsdram, FMC_SDRAM_CommandTypeDef *Command)
{
  __IO uint32_t tmpmrd =0;
  /* Step 3:  Configure a clock configuration enable command */
  Command->CommandMode 			 = FMC_SDRAM_CMD_CLK_ENABLE;
  Command->CommandTarget 		 = FMC_SDRAM_CMD_TARGET_BANK2;
  Command->AutoRefreshNumber 	 = 1;
  Command->ModeRegisterDefinition = 0;

  /* Send the command */
  HAL_SDRAM_SendCommand(hsdram, Command, 0x1000);

  /* Step 4: Insert 100 ms delay */
  HAL_Delay(100);

  /* Step 5: Configure a PALL (precharge all) command */
  Command->CommandMode 			 = FMC_SDRAM_CMD_PALL;
  Command->CommandTarget 	     = FMC_SDRAM_CMD_TARGET_BANK2;
  Command->AutoRefreshNumber 	 = 1;
  Command->ModeRegisterDefinition = 0;

  /* Send the command */
  HAL_SDRAM_SendCommand(hsdram, Command, 0x1000);

  /* Step 6 : Configure a Auto-Refresh command */
  Command->CommandMode 			 = FMC_SDRAM_CMD_AUTOREFRESH_MODE;
  Command->CommandTarget 		 = FMC_SDRAM_CMD_TARGET_BANK2;
  Command->AutoRefreshNumber 	 = 4;
  Command->ModeRegisterDefinition = 0;

  /* Send the command */
  HAL_SDRAM_SendCommand(hsdram, Command, 0x1000);

  /* Step 7: Program the external memory mode register */
  tmpmrd = (uint32_t)SDRAM_MODEREG_BURST_LENGTH_2          |
                     SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL   |
                     SDRAM_MODEREG_CAS_LATENCY_3           |
                     SDRAM_MODEREG_OPERATING_MODE_STANDARD |
                     SDRAM_MODEREG_WRITEBURST_MODE_SINGLE;

  Command->CommandMode = FMC_SDRAM_CMD_LOAD_MODE;
  Command->CommandTarget 		 = FMC_SDRAM_CMD_TARGET_BANK2;
  Command->AutoRefreshNumber 	 = 1;
  Command->ModeRegisterDefinition = tmpmrd;

  /* Send the command */
  HAL_SDRAM_SendCommand(hsdram, Command, 0x1000);

  /* Step 8: Set the refresh rate counter */
  /* (15.62 us x Freq) - 20 */
  /* Set the device refresh counter */
  HAL_SDRAM_ProgramRefreshRate(hsdram, REFRESH_COUNT);

  sdram_init=1;
}

void Fill_Buffer(uint32_t *pBuffer, uint32_t uwBufferLenght, uint32_t uwOffset)
{
  uint32_t tmpIndex = 0;

  /* Put in global buffer different values */
  for (tmpIndex = 0; tmpIndex < uwBufferLenght; tmpIndex++ )
  {
    pBuffer[tmpIndex] = tmpIndex + uwOffset;
  }
}

FATFS SDFatFs;  /* File system object for SD card logical drive */
FIL MyFile;     /* File object */
char SDPath[4]; /* SD card logical drive path */

// This stub function is required by stdlib
extern "C" int _open(const char *name, int flags, int mode)
{
    static_cast<void>(name);
    static_cast<void>(flags);
    static_cast<void>(mode);
    return -1;   // does the stub-return need to be a specific value??
}
uint8_t bufptr=0;
uint8_t* output;

int main(void)
{
	  FRESULT res;                                          /* FatFs function common result code */
	  uint32_t byteswritten, bytesread;                     /* File write/read counts */
	  uint8_t wtext[] = "This is STM32 working with FatFs"; /* File write buffer */
	  uint8_t rtext[100];                                   /* File read buffer */

	hsdram.Instance = FMC_SDRAM_DEVICE;
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();
  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_FMC_Init();
  SDRAM_Initialization_Sequence(&hsdram, &command);
  MX_I2S3_Init();
  MX_RTC_Init();
  MX_SDIO_SD_Init();
  MX_USART1_UART_Init();
  MX_USART6_UART_Init();
  MX_FATFS_Init();

//  while (1) {

//  }

  /*MX_USB_DEVICE_Init();
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11, GPIO_PIN_SET);
  while (1) ;*/


//  MX_USB_HOST_Init();

  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  /* Program the SDRAM external device */


    /*##-2- SDRAM memory read/write access #####################################*/

    /* Fill the buffer to write */
/*
    volatile uint32_t* p = (uint32_t*) 0xd0000000;
    for (uint32_t i=0;i<32*1024*1024/sizeof(uint32_t);i++)
    {
    	*p=0x12345678 ^ i;
    	p++;
    }

    p = (uint32_t*) 0xd0000000;
    for (uint32_t i=0;i<32*1024*1024/sizeof(uint32_t);i++)
    {
    	if (*p != (0x12345678 ^ i)) {
    		uwWriteReadStatus++;
    	}
    	p++;
    }


    if (uwWriteReadStatus)
    {
      //BSP_LED_On(LED4);
    }
    else
    {
      //BSP_LED_On(LED3);
    }
*/

    //##-2- Register the file system object to the FatFs module ##############
        if(f_mount(&SDFatFs, (TCHAR const*)SDPath, 0) != FR_OK)
        {
          // FatFs Initialization Error
          Error_Handler();
        }
		HAL_Delay(200);
		if(f_open(&MyFile, "TURRICAN.SID", FA_READ) != FR_OK)
		{
		  Error_Handler();
		}
		uint32_t size = f_size(&MyFile);

		HAL_Delay(200);

//		sdram_init=1;
		uint8_t *sidfile = (uint8_t*) malloc(size);
//		sdram_init=0;

	  res = f_read(&MyFile, sidfile, size, (UINT*)&bytesread);

	  if((bytesread == 0) || (res != FR_OK))
	  {
		// 'STM32.TXT' file Read or EOF Error
		Error_Handler();
	  }
	  f_close(&MyFile);

	  sidplay2 Sidplay2;

	  SidTune Tune(sidfile, size);
	  ReSIDBuilder ReSID("");
	  Tune.selectSong(1);

	  Sidplay2.load(&Tune);
	  sid2_config_t sid2config;
	  //ReSIDBuilder resid("name");

	  ReSID.create(1);
	  sid2config.frequency = 22050;//22050;
	  sid2config.emulateStereo = true;
	  sid2config.leftVolume = 128;
	  sid2config.rightVolume = 128;
	  sid2config.clockSpeed = SID2_CLOCK_PAL;
	  sid2config.optimisation = 1;
	  sid2config.sampleFormat = SID2_BIG_UNSIGNED;
	  sid2config.sidModel = SID2_MOS6581;
	  sid2config.forceDualSids = false;
	  sid2config.sidSamples = true;
	  sid2config.playback = sid2_stereo;
	  sid2config.environment = sid2_envPS;
	  sid2config.precision = 16;
	  sid2config.sidEmulation = &ReSID;
	  Sidplay2.config(sid2config);


	   // f_open(&MyFile,"JINKS.WAV", FA_WRITE | FA_CREATE_ALWAYS);

//		sdram_init=1;
    output = (uint8_t*) malloc(22050*2*sizeof(uint16_t)*32);
//	sdram_init=0;

    uint32_t tmp=0;
    uint8_t flag=0;
    Sidplay2.play(output, 22050*2*2);
    for (int i=0;;i++) {
    	Sidplay2.play((output+22050*2*sizeof(uint16_t)*(i % 32)), 22050*2*2);
    	if (i==5) {
    		HAL_I2S_Transmit_DMA(&hi2s3,(uint16_t*) output,22050*2);
    	}
//    	flag=1-flag;
//    	while (HAL_I2S_Transmit_DMA(&hi2s3,(uint16_t*) output,22050*2) == HAL_BUSY);

       }
    //f_write(&MyFile, (void*) output, 44100*2*2, (UINT*)&bytesread);

    HAL_Delay(200);
    HAL_Delay(200);
    f_close(&MyFile);


  while (1)
  {
  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
// MX_USB_HOST_Process();
  }
  /* USER CODE END 3 */

}

void HAL_I2S_TxCpltCallback(I2S_HandleTypeDef *hi2s)
{
 /* Prevent unu
  * sed argument(s) compilation warning */
	bufptr++;
	bufptr%=32;
	HAL_I2S_Transmit_DMA(&hi2s3,(uint16_t*) (output+22050*2*sizeof(uint16_t)*(bufptr % 32)),22050*2);

 /* NOTE : This function Should not be modified, when the callback is needed,
           the HAL_I2S_TxCpltCallback could be implemented in the user file
  */
}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct;

    /**Configure the main internal regulator output voltage 
    */
  __HAL_RCC_PWR_CLK_ENABLE();

  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE
                              |RCC_OSCILLATORTYPE_LSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 4;
  RCC_OscInitStruct.PLL.PLLN = 168;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK)
  {
    Error_Handler();
  }

  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_I2S|RCC_PERIPHCLK_RTC;
  PeriphClkInitStruct.PLLI2S.PLLI2SN = 172;
  PeriphClkInitStruct.PLLI2S.PLLI2SR = 7;
  PeriphClkInitStruct.RTCClockSelection = RCC_RTCCLKSOURCE_LSE;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
